@@ -677,7 +677,7 @@ public abstract class AbstractProblemDetector implements IApiProblemDetector {
 			String name = apitype.getSimpleName();
 			ICompilationUnit cunit = type.getCompilationUnit();
 			if (cunit.isWorkingCopy()) {
-				cunit.reconcile(AST.JLS14, false, null, null);
+				cunit.reconcile(AST.JLS15, false, null, null);
 			}
 			IMethod method = getEnclosingMethod(type, reference, doc);
 			if (method != null) {
@@ -735,7 +735,7 @@ public abstract class AbstractProblemDetector implements IApiProblemDetector {
 				if (method == null) {
 					// look it up the hard way
 					ISourceRange range = jtype.getCompilationUnit().getSourceRange();
-					ASTParser parser = ASTParser.newParser(AST.JLS14);
+					ASTParser parser = ASTParser.newParser(AST.JLS15);
 					parser.setSource(jtype.getCompilationUnit());
 					parser.setSourceRange(range.getOffset(), range.getLength());
 					parser.setResolveBindings(true);
@@ -946,7 +946,7 @@ public abstract class AbstractProblemDetector implements IApiProblemDetector {
 				// what if space between the two?
 				start = line.indexOf(METHOD_REFERENCE + CONSTRUCTOR_NEW);
 				if (start < 0) {
-					line.indexOf(CONSTRUCTOR_NEW);
+					start = line.indexOf(CONSTRUCTOR_NEW);
 					if (start < 0) {
 						start = 0;
 					}
