@@ -20,32 +20,38 @@ import java.util.*;
 import org.eclipse.swt.internal.*;
 
 // Common type translation table:
-// C   ->  Java
+// C            ->  Java
 // --------------------
 // Primitives:
-// int   -> int
-// guint -> long   #Reason:
-//					c : unsigned int range: 4294967295
-//                  java : int range      : 2147483647 (less than c unsigned int)
-//                  Java : long range: 9,223,372,036,854,775,807
-//				    // Note: Not to be used for pointers.
+// int          -> int
+// gint*        -> int[]
 //
-// gint* -> int[]
-// boolean   -> int  ex setenv
-// gboolean  -> boolean
+// Unsigned integer:
+// * Note that java's int is signed, which introduces difficulties
+// * for values > 0x7FFFFFFF. Java's long can fit such values, but
+// * java's long is 8 bytes, while guint is 4 bytes. For that reason,
+// * java's long CAN'T be used for pointers or arrays.
+// guint        -> int/long
+// guint*       -> int[]
+//
+// Boolean:
+// * Java's boolean is handy, but it's 1 byte, while gboolean is 4
+// * bytes. For that reason, it CAN'T be used for pointers or arrays.
+// gboolean     -> int/boolean
+// gboolean*    -> int
 //
 // Pointers:
-// gpointer -> long
-// void *   -> long
+// gpointer     -> long
+// void *       -> long
 //
 // Strings:
-// gchar *      -> long
-// const char * -> byte[]  ex setenv
-// const gchar* -> byte[]  ex g_log_remove_handler
+// gchar *      -> long    // You're responsible for allocating/deallocating memory buffer.
+// const char * -> byte[]  // Example: setenv()
+// const gchar* -> byte[]  // Example: g_log_remove_handler()
 //
 // Special types:
-// GQuark -> int
-// GError ** -> long []  ex g_filename_to_uri
+// GQuark       -> int
+// GError **    -> long[]  // Example: g_filename_to_uri()
 
 
 /**
@@ -348,6 +354,7 @@ public class OS extends C {
 	public static final byte[] show = ascii("show");
 	public static final byte[] show_help = ascii("show-help");
 	public static final byte[] size_allocate = ascii("size-allocate");
+	public static final byte[] resize = ascii("resize");
 	public static final byte[] start_interactive_search = ascii("start-interactive-search");
 	public static final byte[] style_updated = ascii("style-updated");
 	public static final byte[] switch_page = ascii("switch-page");
@@ -359,6 +366,8 @@ public class OS extends C {
 	public static final byte[] value_changed = ascii("value-changed");
 	public static final byte[] window_state_event = ascii("window-state-event");
 	public static final byte[] notify_state = ascii("notify::state");
+	public static final byte[] notify_theme_change = ascii("notify::gtk-application-prefer-dark-theme");
+	public static final byte[] response = ascii("response");
 
 	/** Properties */
 	public static final byte[] active = ascii("active");
@@ -395,6 +404,10 @@ public class OS extends C {
 	public static final byte[] margin_top = ascii("margin-top");
 	public static final byte[] scrollbar_spacing = ascii("scrollbar-spacing");
 
+	/** Actions */
+	public static final byte[] action_copy_clipboard = ascii("clipboard.copy");
+	public static final byte[] action_cut_clipboard = ascii("clipboard.cut");
+	public static final byte[] action_paste_clipboard = ascii("clipboard.paste");
 
 	/** CUSTOM_CODE START
 	 *
@@ -494,6 +507,19 @@ public class OS extends C {
 	 */
 	public static final native void swt_fixed_resize(long fixed, long widget, int width, int height);
 
+	/**
+	 * @param container cast=(GtkWidget*)
+	 * @param widget cast=(GtkWidget*)
+	 * @category custom
+	 */
+	public static final native void swt_fixed_add(long container, long widget);
+	/**
+	 * @param container cast=(GtkWidget*)
+	 * @param widget cast=(GtkWidget*)
+	 * @category custom
+	 */
+	public static final native void swt_fixed_remove(long container, long widget);
+
 	/** @param str cast=(const gchar *)
 	 * @category custom
 	 */
@@ -2251,4 +2277,83 @@ public static final native void g_variant_unref(long value);
  * @param object cast=(GObject *)
  */
 public static final native long g_object_ref_sink(long object);
+
+/**
+ * @param dateTime cast=(GDateTime *)
+ * @param year cast=(gint *)
+ * @param month cast=(gint *)
+ * @param day cast=(gint *)
+ */
+public static final native void g_date_time_get_ymd(long dateTime, int[] year, int[] month, int[] day);
+
+public static final native long g_date_time_new_local(int year, int month, int day, int hour, int minute, double seconds);
+
+/** @param file cast=(GFile *) */
+public static final native long g_file_get_path(long file);
+
+
+/* GMenu */
+public static final native long g_menu_new();
+/**
+ * @param label cast=(const gchar *)
+ * @param submenu cast=(GMenuModel *)
+ */
+public static final native long g_menu_item_new_submenu(byte[] label, long submenu);
+/**
+ * @param label cast=(const gchar *)
+ * @param section cast=(GMenuModel *)
+ */
+public static final native long g_menu_item_new_section(byte[] label, long section);
+/**
+ * @param label cast=(const gchar *)
+ * @param detailed_action cast=(const gchar *)
+ */
+public static final native long g_menu_item_new(byte[] label, byte[] detailed_action);
+/**
+ * @param menu_item cast=(GMenuItem *)
+ * @param submenu cast=(GMenuModel *)
+ */
+public static final native void g_menu_item_set_submenu(long menu_item, long submenu);
+/**
+ * @param menu cast=(GMenu *)
+ * @param item cast=(GMenuItem *)
+ */
+public static final native void g_menu_insert_item(long menu, int position, long item);
+/** @param menu cast=(GMenu *) */
+public static final native void g_menu_remove(long menu, int position);
+/**
+ * @param menu_item cast=(GMenuItem *)
+ * @param label cast=(const gchar *)
+ */
+public static final native void g_menu_item_set_label(long menu_item, byte[] label);
+/**
+ * @param menu_item cast=(GMenuItem *)
+ * @param attribute cast=(const gchar *)
+ * @param format_string cast=(const gchar *)
+ * @param data cast=(const gchar *)
+ */
+public static final native void g_menu_item_set_attribute(long menu_item, byte[] attribute, byte[] format_string, long data);
+
+
+/* GMenuModel */
+/** @param model cast=(GMenuModel *) */
+public static final native int g_menu_model_get_n_items(long model);
+
+
+/* GAction */
+public static final native long g_simple_action_group_new();
+/**
+ * @param name cast=(const gchar *)
+ * @param parameter_type cast=(const GVariantType *)
+ */
+public static final native long g_simple_action_new(byte[] name, long parameter_type);
+/**
+ * @param action_map cast=(GActionMap *)
+ * @param action cast=(GAction *)
+ */
+public static final native void g_action_map_add_action(long action_map, long action);
+/** @param action cast=(GAction *) */
+public static final native boolean g_action_get_enabled(long action);
+/** @param simple_action cast=(GSimpleAction *) */
+public static final native void g_simple_action_set_enabled(long simple_action, boolean enabled);
 }
